# pylint: disable=invalid-name

import pytest
from pytest_data import get_data

from .models import HostingVideoPlayer, SourceFileVideoPlayer


@pytest.fixture
def cms_qe_video_source_file_video_player_model(request):
    return SourceFileVideoPlayer(**get_data(
        request,
        'cms_qe_video_source_file_video_player_model_data',
    ))


@pytest.fixture
def cms_qe_video_hosting_video_player_model(request):
    return HostingVideoPlayer(**get_data(
        request,
        'cms_qe_video_hosting_video_player_model_data',
    ))



import re
from django.core import mail
from pytest_data import use_data

from cms_qe_auth.models import User
from cms_qe_auth.utils import pk_to_uidb64


def test_set_username_by_email():
    user = User(email='user@example.com')
    assert not user.username
    user.save()
    assert user.username == user.email


@use_data(user_data={'username': 'user@example.com', 'email': 'user@example.com'})
def test_set_username_by_changing_email(user):
    assert user.username == 'user@example.com'
    user.email = 'another@example.com'
    assert user.username == 'user@example.com'
    user.save()
    assert user.username == 'another@example.com'


@use_data(user_data={'email': 'user@example.com'})
def test_unset_is_active_by_changing_email(user):
    assert user.is_active
    user.email = 'another@example.com'
    assert user.is_active
    user.save()
    assert not user.is_active


def test_simple_token_generation_and_checking(user):
    token = user._generate_activation_token()
    assert user._check_activation_token(token)


@use_data(user_data={'is_active': False})
def test_simple_token_generation_and_activate_is_active(user):
    assert not user.is_active
    token = user._generate_activation_token()
    user.activate(token)
    assert user.is_active


@use_data(user_data={'is_active': False})
def test_simple_token_generation_and_activate_invalid(user):
    user._generate_activation_token()
    assert not user.activate("123456789ABZ-987654321abcdefg")
    assert not user.is_active


def test_activation_email_sending(user):
    assert len(mail.outbox) == 0
    user.send_activation_email(base_url='https://test.com')
    assert len(mail.outbox) == 1

    email = mail.outbox[0]
    assert user.email == email.to[0]
    assert 'activate' in email.subject.lower()

    token_re = r'(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})'
    url_re = r'//test.com/en/auth/activate/{}/{}'.format(pk_to_uidb64(user.pk).decode('utf-8'), token_re)
    match = re.findall(url_re, email.body)
    assert match
    assert user._check_activation_token(match[0])



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-20 09:54
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
    ]

    operations = [
        migrations.CreateModel(
            name='MenuPluginModel',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='cms_qe_menu_menupluginmodel', serialize=False, to='cms.CMSPlugin')),
                ('orientation', models.IntegerField(choices=[(1, 'Horizontal'), (2, 'Vertical')], default=1)),
                ('start_page', models.CharField(blank=True, help_text='ID of page in advanced settings.', max_length=50)),
                ('end_level', models.IntegerField(default=2, help_text='\n            Specify at which level the navigation should stop.\n\n            Keep in mind that Bootstrap by default does not support dropdown submenu.\n            Any deep dropdown menu is not very well accessible from mobile devices.\n        ', verbose_name='End level')),
                ('submenu_in_inactive_item', models.BooleanField(default=True, verbose_name='Submenu for inactive items')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



from cms_qe_newsletter.models import Subscriber
from django import forms


class SubscriberForm(forms.ModelForm):
    """
    Subscribers form validates subscriber data.
    """

    class Meta:
        model = Subscriber
        fields = ('email', 'first_name', 'last_name')

    def __init__(self, fullname_require, *args, **kwargs):
        super(SubscriberForm, self).__init__(*args, **kwargs)

        self.fullname_require = fullname_require
        if fullname_require:
            self.fields['first_name'].required = True
            self.fields['last_name'].required = True

    # pylint: disable=arguments-differ
    def save(self, mailing_lists, commit=True):
        """
        Add a new subscriber to all mail lists which was selected in the plugin settings.
        """
        data = self.cleaned_data
        email = data['email']
        first_name = data['first_name']
        last_name = data['last_name']
        for mailing_list in mailing_lists:
            if not Subscriber.objects.filter(email=email, mailing_list=mailing_list).count():
                Subscriber.objects.create(
                    mailing_list=mailing_list,
                    email=email,
                    first_name=first_name,
                    last_name=last_name,
                )



"""
Piwik template tags and filters.
"""

from __future__ import absolute_import

from collections import namedtuple
from itertools import chain
import re

from django.template import Library, Node, TemplateSyntaxError

from ..utils import (is_internal_ip, disable_html,
                     get_required_setting, get_identity, AnalyticalNotSet)

# domain name (characters separated by a dot), optional port, optional URI path, no slash
DOMAINPATH_RE = re.compile(r'^(([^./?#@:]+\.)*[^./?#@:]+)+(:[0-9]+)?(/[^/?#@:]+)*$')

# numeric ID
SITEID_RE = re.compile(r'^\d+$')

TRACKING_CODE = """
<script type="text/javascript">
  var _paq = _paq || [];
  %(variables)s
  _paq.push(['trackPageView']);
  _paq.push(['enableLinkTracking']);
  (function() {
    var u=(("https:" == document.location.protocol) ? "https" : "http") + "://%(url)s/";
    _paq.push(['setTrackerUrl', u+'piwik.php']);
    _paq.push(['setSiteId', %(siteid)s]);
    var d=document, g=d.createElement('script'), s=d.getElementsByTagName('script')[0]; g.type='text/javascript';
    g.defer=true; g.async=true; g.src=u+'piwik.js'; s.parentNode.insertBefore(g,s);
  })();
</script>
<noscript><p><img src="http://%(url)s/piwik.php?idsite=%(siteid)s" style="border:0;" alt="" /></p></noscript>
"""  # noqa

VARIABLE_CODE = '_paq.push(["setCustomVariable", %(index)s, "%(name)s", "%(value)s", "%(scope)s"]);'  # noqa
IDENTITY_CODE = '_paq.push(["setUserId", "%(userid)s"]);'

DEFAULT_SCOPE = 'page'

PiwikVar = namedtuple('PiwikVar', ('index', 'name', 'value', 'scope'))

register = Library()


# pylint:disable=unused-argument
@register.tag
def piwik(parser, token):
    """
    Piwik tracking template tag.

    Renders Javascript code to track page visits.  You must supply
    your Piwik domain (plus optional URI path), and tracked site ID
    in the ``PIWIK_DOMAIN_PATH`` and the ``PIWIK_SITE_ID`` setting.

    Custom variables can be passed in the ``piwik_vars`` context
    variable.  It is an iterable of custom variables as tuples like:
    ``(index, name, value[, scope])`` where scope may be ``'page'``
    (default) or ``'visit'``.  Index should be an integer and the
    other parameters should be strings.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return PiwikNode()


class PiwikNode(Node):
    def __init__(self):
        try:
            self.domain_path = \
                get_required_setting('PIWIK_DOMAIN_PATH', DOMAINPATH_RE,
                                     "must be a domain name, optionally followed "
                                     "by an URI path, no trailing slash (e.g. "
                                     "piwik.example.com or my.piwik.server/path)")
            self.site_id = \
                get_required_setting('PIWIK_SITE_ID', SITEID_RE,
                                     "must be a (string containing a) number")
            self.id_and_domain_is_set = True
        except AnalyticalNotSet:
            self.id_and_domain_is_set = False

    def render(self, context):
        if not self.id_and_domain_is_set:
            return ''

        custom_variables = context.get('piwik_vars', ())

        complete_variables = (var if len(var) >= 4 else var + (DEFAULT_SCOPE,)
                              for var in custom_variables)

        variables_code = (VARIABLE_CODE % PiwikVar(*var)._asdict()
                          for var in complete_variables)

        userid = get_identity(context, 'piwik')
        if userid is not None:
            variables_code = chain(variables_code, (
                IDENTITY_CODE % {'userid': userid},
            ))

        html = TRACKING_CODE % {
            'url': self.domain_path,
            'siteid': self.site_id,
            'variables': '\n  '.join(variables_code)
        }
        if is_internal_ip(context, 'PIWIK'):
            html = disable_html(html, 'Piwik')
        return html



from typing import Iterable

from constance import config


# pylint:disable=global-variable-undefined
class MailchimpIsNotSetException(Exception):
    pass


class MailChimpClient:
    """
    Helper class to communicate with MailChimp.
    """

    def __init__(self) -> None:
        """
        Checking if MailChimp API key and username is configured and return a MailChimp API client.
        """
        username = config.MAILCHIMP_USERNAME
        api_key = config.MAILCHIMP_API_KEY
        if api_key and username:
            # Dynamic import so it's possible to mock.
            from mailchimp3 import MailChimp
            self.client = MailChimp(username, api_key)
        else:
            raise MailchimpIsNotSetException('API key or username for mailchimp is not set.')

    def get_lists(self) -> Iterable[str]:
        """
        :return: List of lists with fields ``id`` and ``name``.
        """
        return self.client.lists.all(get_all=True, fields='lists.name,lists.id')

    def subscribe(self, mailing_list_id, email, first_name, last_name) -> None:
        """
        Add a subscriber and return the id (subscriber_hash).
        """
        new_subscriber = self.client.lists.members.create(mailing_list_id, {
            'email_address': email,
            'status': 'subscribed',
            'merge_fields': {
                'FNAME': first_name,
                'LNAME': last_name,
            },
        })
        return new_subscriber['id']

    def unsubscribe(self, mailing_list_id, subscriber_hash) -> None:
        """
        Remove a subscriber.
        """
        self.client.lists.members.delete(mailing_list_id, subscriber_hash)



"""
Mailing settings, by default app looks for smtp server.
"""

EMAIL_HOST = 'smtp'
EMAIL_PORT = 587

EMAIL_HOST_USER = ''
EMAIL_HOST_PASSWORD = ''

EMAIL_USE_TLS = True
EMAIL_SUBJECT_PREFIX = ''

DEFAULT_FROM_EMAIL = 'django_cms_qe@localhost'




import json
import logging

from django.http import HttpRequest, HttpResponse
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST

__all__ = ('csp_report',)


@csrf_exempt
@require_POST
def csp_report(request: HttpRequest) -> HttpResponse:
    """
    View handling reports by CSP headers. When there is problem by CSP,
    then browser fire request to this view with JSON data describing
    problem. It's simply just logged as warning for later analyzing.
    """
    data = request.read()
    data = json.loads(str(data, 'utf8', 'replace'))
    logging.warning(data)
    return HttpResponse('OK')



from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from cms_qe_video.forms import HostingVideoPlayerForm
from django.utils.translation import ugettext_lazy as _

from . import models


@plugin_pool.register_plugin
class SourceFileVideoPlayerPlugin(CMSPluginBase):
    model = models.SourceFileVideoPlayer
    module = _('Video player')
    name = _('Video player - source file')
    render_template = 'cms_qe/video/video_source_file.html'
    text_enabled = True
    allow_children = True
    child_classes = ['VideoTrackPlugin']
    fieldsets = [
        (None, {
            'fields': (
                'label',
            )
        }),
        (_('Source file settings'), {
            'fields': (
                'source_file',
                'text_title',
            )
        }),

        (_('Player settings'), {
            'classes': ('collapse',),
            'fields': (
                'width',
                'height',
                'controls',
                'autoplay',
                'loop',
                'muted',
                'other_attributes',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'poster',
                'other_attributes',
                'text_description',
            )
        })
    ]


@plugin_pool.register_plugin
class HostingVideoPlayerPlugin(CMSPluginBase):
    model = models.HostingVideoPlayer
    module = _('Video player')
    name = _('Video player - hosting services')
    text_enabled = True
    render_template = 'cms_qe/video/iframe_with_embed_link.html'
    change_form_template = 'cms_qe/video/video_widget.html'
    form = HostingVideoPlayerForm

    fieldsets = [
        (None, {
            'fields': (
                'label',
                'video_hosting_service',
                'video_url',
            )
        }),
        (_('Player settings'), {
            'classes': ('collapse',),
            'fields': (
                'width',
                'height',
                'controls',
                'autoplay',
                'loop',
                'other_attributes',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'poster',
                'other_attributes',
            )
        })
    ]


@plugin_pool.register_plugin
class VideoTrackPlugin(CMSPluginBase):
    model = models.VideoTrack
    module = _('Video player')
    name = _('Track')
    render_template = 'cms_qe/video/track.html'
    require_parent = True
    parent_classes = ['SourceFileVideoPlayerPlugin']

    fieldsets = [
        (None, {
            'fields': (
                'kind',
                'src',
                'srclang',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'label',
                'attributes',
            )
        })
    ]



from typing import Dict, List, Tuple

from django.apps import apps
from django.db.models.base import ModelBase
from django.contrib.auth import get_user_model

from .exceptions import TableDoesNotExists


def get_model_by_table(table: str) -> ModelBase:
    """
    Returns Django model by table name.
    """
    if table == 'auth_user':
        return get_user_model()

    for app_models in apps.all_models.values():
        for cls in app_models.values():
            if cls._meta.db_table == table:
                return cls
    raise TableDoesNotExists(table)


def get_models_choices() -> Tuple[Tuple[str, Tuple[Tuple[str, str], ...]], ...]:
    """
    Returns sorted and grouped choices of all models per app.

    Example output:

    .. code-block:: txt

        (
            'app1',
            (
                ('choice1 key', 'choice1 name'),
                ...
            ),
        ), (
            'app2',
            (
                ('choice2 key', 'choice2 name'),
                ...
            ),
        ), (
            ...
        )
    """
    return tuple((
        app,
        tuple(
            (cls._meta.db_table, cls._meta.object_name)
            for model, cls in sorted(app_models.items())
        ),
    ) for app, app_models in sorted(apps.all_models.items()) if app_models)


def get_table_choices(table: str) -> Dict[str, List[Tuple[str, str]]]:
    """
    Returns other choices for table depending on exact table name.

    .. code-block:: txt

        {
            "columns": [
                ["name", "label"],
                ...
            ]
        }
    """
    model = get_model_by_table(table)
    choices = {
        'columns': [(field.name, getattr(field, 'verbose_name', field.name)) for field in model._meta.get_fields()],
    }
    return choices



from ..templatetags.cms_qe_video import cms_qe_video_url_to_embed
import pytest


@pytest.mark.parametrize('url, expected', [
    ('https://servis.com/somevideo', 'https://servis.com/somevideo'),
    ('http://vimeo.com/somevideo', 'http://player.vimeo.com/video/somevideo'),
    ('https://vimeo.com/somevideo', 'https://player.vimeo.com/video/somevideo'),
    ('https://vimeo.com/somevideo?param', 'https://player.vimeo.com/video/somevideo'),
    ('https://youtu.be/watch?v=ZGuQmszmtaQ', 'https://www.youtube.com/embed/ZGuQmszmtaQ'),
    ('https://www.youtube.com/watch?v=ZGuQmszmtaQ', 'https://www.youtube.com/embed/ZGuQmszmtaQ'),
    # Test for bug with additional parameters in link.
    # Related on issue: https://gitlab.labs.nic.cz/websites/django-cms-qe/issues/41
    ('https://www.youtube.com/watch?v=ZGuQmszmtaQ&index=10&list=PLfTu7SiuiT_izjvg_1JRKXkrWSnvuP4pd',
     'https://www.youtube.com/embed/ZGuQmszmtaQ'),
])
def test_url_to_embed(url, expected):
    assert expected == cms_qe_video_url_to_embed(url)



from django.conf import settings
from django.core.cache import cache
from django.db import connections
from django.db.utils import OperationalError


def get_status():
    return {
        'database': check_database(),
        'cache': check_cache(),
    }


def check_database():
    db_conn = connections['default']
    try:
        db_conn.cursor()
    except OperationalError:
        connected = False
    else:
        connected = True
    return connected


def check_cache():
    if 'DummyCache' in settings.CACHES['default']['BACKEND']:
        return True

    try:
        cache.set('_cms_qe_monitoring', 'test', 5)
        return cache.get('_cms_qe_monitoring') == 'test'
    except Exception as exc:  # pylint: disable=broad-except
        return str(exc)



from django.contrib.auth import get_user_model
import pytest

from ..exceptions import TableDoesNotExists
from ..utils import get_model_by_table, get_models_choices, get_table_choices


def test_get_model_by_table():
    User = get_user_model()
    model = get_model_by_table('auth_user')
    assert model is User


def test_get_model_by_table_not_found():
    with pytest.raises(TableDoesNotExists):
        get_model_by_table('table_does_not_exist')


def test_get_all_models():
    choices = get_models_choices()
    choices_admin_group = [item[1] for item in choices if item[0] == 'admin'][0]
    assert choices_admin_group == (
        ('django_admin_log', 'LogEntry'),
    )


def test_get_table_choices():
    choices = get_table_choices('auth_user')
    assert 'columns' in choices
    assert ('username', 'username') in choices['columns']



from django.core.management.base import BaseCommand

from cms_qe_newsletter import logger
from cms_qe_newsletter.external_services.sync import sync_tasks


class Command(BaseCommand):
    """
    Command processes the queue to subscribing and unsubscribing
    on the external services. Usage of command with ``manage.py``::

        python -m manage.py cms_qe_newsletter_sync
    """

    def handle(self, *args, **options):
        logger.info('Newsletter sync started...')

        for task_result, task_message in sync_tasks():
            if task_result is None:
                logger.warning(task_message)
            elif task_result is False:
                logger.error(task_message)
            else:
                logger.info(task_message)

        logger.info('Newsletter sync finished...')



from django.contrib.staticfiles.storage import ManifestStaticFilesStorage as DjangoManifestStaticFilesStorage


class ManifestStaticFilesStorage(DjangoManifestStaticFilesStorage):
    """
    Custom handle of ``django.contrib.staticfiles.storage.ManifestStaticFilesStorage``
    to create different URLs for every version of static file. Means when you change
    static file, Django's ``collectstatic`` detects that and creates ``staticfiles.json``
    with all hashes and adds them to URLs when tag ``static`` is used.

    It's good to use this when you don't want to have problems with caches--when you
    change static but cache still serves the old one with new generated HTML. Thanks
    to this storage you can be sure that every client will use new resources needed
    by new page.

    This custom version takes care of compatiblity of Django CMS which brings custom
    static tag to add owns version and then Django's storage has problem to cooperate
    with it. In Django 1.10 it has to override ``stored_name`` to ignore this problem
    and in Django 1.11 is brought new attribute ``manifest_strict`` which has to be
    set to ``False`` to work as in Django 1.10.
    """

    manifest_strict = False

    def stored_name(self, name):
        try:
            return super().stored_name(name)
        except ValueError:
            return name



from selenium.common.exceptions import NoSuchElementException
from webdriverwrapper import Chrome as _Chrome


class ChromeBrowser(_Chrome):
    def get_error_page(self):
        # Django error page.
        try:
            traceback = self.get_elm('traceback')
            error_page = self.get_elm('summary')
        except NoSuchElementException:
            pass
        else:
            header = error_page.get_elm(tag_name='h1')
            return header.text

        # Generic error page.
        try:
            error_page = self.get_elm('error-page')
        except NoSuchElementException:
            pass
        else:
            header = error_page.get_elm(tag_name='h1')
            return header.text

    def get_error_traceback(self):
        try:
            traceback = self.get_elm('traceback_area')
        except NoSuchElementException:
            pass
        else:
            return traceback.text

    def get_error_messages(self):
        try:
            error_elms = self.get_elms(xpath='//*[contains(@class, "alert-danger")]')
        except NoSuchElementException:
            return []
        else:
            return [error_elm.text for error_elm in error_elms]

    def get_info_messages(self):
        try:
            info_elms = self.get_elms(xpath='//*[contains(@class, "alert-success")]')
        except NoSuchElementException:
            return []
        else:
            return [info_elm.get_attribute('info') for info_elm in info_elms]



from pytest_data import use_data

from ..models import SubscribeTask
from ..external_services.sync import sync_tasks, sync_task


@use_data(
    cms_qe_mailing_list_data={'external_service': 99},
    cms_qe_subscribe_task_data={
        'attempts': 0,
        'last_error': 0,
    },
)
def test_save_failure(cms_qe_subscribe_task):
    assert SubscribeTask.objects.all().count() == 1
    result, message = list(sync_tasks())[0]
    assert 'Unsupported service 99' in message
    assert result is False
    cms_qe_subscribe_task.refresh_from_db()
    assert cms_qe_subscribe_task.attempts == 1
    assert cms_qe_subscribe_task.last_error == 'Unsupported service 99'


@use_data(cms_qe_subscribe_task_data={'attempts': 10})
def test_warning_do_not_increment_failure(cms_qe_subscribe_task):
    assert SubscribeTask.objects.all().count() == 1
    result, message = list(sync_tasks())[0]
    assert 'Skipped' in message
    assert result is None
    cms_qe_subscribe_task.refresh_from_db()
    assert cms_qe_subscribe_task.attempts == 10
    assert cms_qe_subscribe_task.last_error == ''


@use_data(cms_qe_subscribe_task_data={'attempts': 10})
def test_task_skip(cms_qe_subscribe_task):
    result, message = sync_task(cms_qe_subscribe_task)
    assert 'Skipped' in message
    assert result is None


@use_data(
    cms_qe_mailing_list_data={'external_service': 99},
    cms_qe_subscribe_task_data={
        'attempts': 0,
        'last_error': 0,
    },
)
def test_task_unsupported_service(cms_qe_subscribe_task):
    result, message = sync_task(cms_qe_subscribe_task)
    assert 'Unsupported service 99' in message
    assert result is False


@use_data(
    cms_qe_subscribe_task_data={'email': '-no-subscriber-@example.com'},
)
def test_task_with_removed_subscriber(cms_qe_subscribe_task):
    result, message = sync_task(cms_qe_subscribe_task)
    assert 'Subscriber does not exist anymore, deleting task' in message
    assert result is None


@use_data(
    cms_qe_subscriber_data={'email': 'test@example.com'},
    cms_qe_subscribe_task_data={'email': 'test@example.com'},
)
def test_task_synced(cms_qe_subscribe_task, cms_qe_subscriber, mock_mailchimp):
    result, message = sync_task(cms_qe_subscribe_task)
    assert 'OK' in message
    assert result is True



import csv
import io
import json

from django.contrib.auth import get_user_model
import pytest
from pytest_data import use_data

from ..export import export_data


@pytest.mark.django_db
@use_data(user_data={'username': 'test_export_data_as_csv'})
def test_export_data_as_csv(user):
    User = get_user_model()
    queryset = User.objects.all()
    data = export_data('csv', queryset)
    data = list(csv.reader(io.StringIO(data)))
    assert len(data) == 2
    assert 'test_export_data_as_csv' in data[1]


@pytest.mark.django_db
@use_data(user_data={'username': 'test_export_data_as_tsv'})
def test_export_data_as_tsv(user):
    User = get_user_model()
    queryset = User.objects.all()
    data = export_data('tsv', queryset)
    data = list(csv.reader(io.StringIO(data), delimiter='\t'))
    assert len(data) == 2
    assert 'test_export_data_as_tsv' in data[1]


@pytest.mark.django_db
@use_data(user_data={'username': 'test_export_data_as_json'})
def test_export_data_as_json(user):
    User = get_user_model()
    queryset = User.objects.all()
    data = export_data('json', queryset)
    data = json.loads(data)
    assert len(data) == 1
    assert data[0]['username'] == 'test_export_data_as_json'


@pytest.mark.django_db
@use_data(user_data={'username': 'test_export_data_as_yaml'})
def test_export_data_as_yaml(user):
    User = get_user_model()
    queryset = User.objects.all()
    data = export_data('yaml', queryset)
    assert 'test_export_data_as_yaml' in data


@pytest.mark.django_db
@use_data(user_data={'username': 'test_export_data_as_html'})
def test_export_data_as_html(user):
    User = get_user_model()
    queryset = User.objects.all()
    data = export_data('html', queryset)
    assert '<table>' in data
    assert 'test_export_data_as_html' in data



from django import template

register = template.Library()


@register.filter
def add_str(value, arg):
    """
    Same as :py:func:`django.template.defaultfilters.add` but always convert to ``str``.
    """
    return str(value) + str(arg)



# pylint: disable=all

# This is copy of some django.forms.widget to assign removed
# RadioFieldRenderer in 1.11 to work properly with Aldryn
# until they support Django 1.11 as well.

from django.forms.utils import flatatt
from django.utils.encoding import (
    force_text, python_2_unicode_compatible,
)
from django.utils.html import format_html, html_safe
from django.utils.safestring import mark_safe


@html_safe
@python_2_unicode_compatible
class SubWidget(object):
    """
    Some widgets are made of multiple HTML elements -- namely, RadioSelect.
    This is a class that represents the "inner" HTML element of a widget.
    """
    def __init__(self, parent_widget, name, value, attrs, choices):
        self.parent_widget = parent_widget
        self.name, self.value = name, value
        self.attrs, self.choices = attrs, choices

    def __str__(self):
        args = [self.name, self.value, self.attrs]
        if self.choices:
            args.append(self.choices)
        return self.parent_widget.render(*args)


@html_safe
@python_2_unicode_compatible
class ChoiceInput(SubWidget):
    """
    An object used by ChoiceFieldRenderer that represents a single
    <input type='$input_type'>.
    """
    input_type = None  # Subclasses must define this

    def __init__(self, name, value, attrs, choice, index):
        self.name = name
        self.value = value
        self.attrs = attrs
        self.choice_value = force_text(choice[0])
        self.choice_label = force_text(choice[1])
        self.index = index
        if 'id' in self.attrs:
            self.attrs['id'] += "_%d" % self.index

    def __str__(self):
        return self.render()

    def render(self, name=None, value=None, attrs=None):
        if self.id_for_label:
            label_for = format_html(' for="{}"', self.id_for_label)
        else:
            label_for = ''
        attrs = dict(self.attrs, **attrs) if attrs else self.attrs
        return format_html(
            '<label{}>{} {}</label>', label_for, self.tag(attrs), self.choice_label
        )

    def is_checked(self):
        return self.value == self.choice_value

    def tag(self, attrs=None):
        attrs = attrs or self.attrs
        final_attrs = dict(attrs, type=self.input_type, name=self.name, value=self.choice_value)
        if self.is_checked():
            final_attrs['checked'] = 'checked'
        return format_html('<input{} />', flatatt(final_attrs))

    @property
    def id_for_label(self):
        return self.attrs.get('id', '')


class RadioChoiceInput(ChoiceInput):
    input_type = 'radio'  # type: ignore

    def __init__(self, *args, **kwargs):
        super(RadioChoiceInput, self).__init__(*args, **kwargs)
        self.value = force_text(self.value)


@html_safe
@python_2_unicode_compatible
class ChoiceFieldRenderer(object):
    """
    An object used by RadioSelect to enable customization of radio widgets.
    """

    choice_input_class = None
    outer_html = '<ul{id_attr}>{content}</ul>'
    inner_html = '<li>{choice_value}{sub_widgets}</li>'

    def __init__(self, name, value, attrs, choices):
        self.name = name
        self.value = value
        self.attrs = attrs
        self.choices = choices

    def __getitem__(self, idx):
        return list(self)[idx]

    def __iter__(self):
        for idx, choice in enumerate(self.choices):
            yield self.choice_input_class(self.name, self.value, self.attrs.copy(), choice, idx)

    def __str__(self):
        return self.render()

    def render(self):
        """
        Outputs a <ul> for this set of choice fields.
        If an id was given to the field, it is applied to the <ul> (each
        item in the list will get an id of `$id_$i`).
        """
        id_ = self.attrs.get('id')
        output = []
        for i, choice in enumerate(self.choices):
            choice_value, choice_label = choice
            if isinstance(choice_label, (tuple, list)):
                attrs_plus = self.attrs.copy()
                if id_:
                    attrs_plus['id'] += '_{}'.format(i)
                sub_ul_renderer = self.__class__(
                    name=self.name,
                    value=self.value,
                    attrs=attrs_plus,
                    choices=choice_label,
                )
                sub_ul_renderer.choice_input_class = self.choice_input_class
                output.append(format_html(
                    self.inner_html, choice_value=choice_value,
                    sub_widgets=sub_ul_renderer.render(),
                ))
            else:
                w = self.choice_input_class(self.name, self.value, self.attrs.copy(), choice, i)
                output.append(format_html(self.inner_html, choice_value=force_text(w), sub_widgets=''))
        return format_html(
            self.outer_html,
            id_attr=format_html(' id="{}"', id_) if id_ else '',
            content=mark_safe('\n'.join(output)),
        )


class RadioFieldRenderer(ChoiceFieldRenderer):
    choice_input_class = RadioChoiceInput  # type: ignore


from django.forms import widgets
widgets.RadioFieldRenderer = RadioFieldRenderer



import pytest
from cms_qe_video.models import VIMEO, YOUTUBE
from django.core.exceptions import ValidationError
from pytest_data import use_data

non_default_data = {
    'controls': False,
    'width': 500,
    'height': 500,
    'autoplay': True,
    'loop': True,
    'muted': True,
}


def test__get_attributes_str_to_html_with_default_attributes(cms_qe_video_source_file_video_player_model):
    attr_str = cms_qe_video_source_file_video_player_model._get_attributes_str_to_html(
        ('width', 'height', 'controls', 'autoplay', 'loop', 'muted'))
    assert 'width=500' not in attr_str and 'height=500' not in attr_str
    assert 'controls' in attr_str
    assert 'autoplay' not in attr_str
    assert 'loop' not in attr_str
    assert 'muted' not in attr_str


@use_data(cms_qe_video_source_file_video_player_model_data=non_default_data)
def test__get_attributes_str_to_html_with_non_default_attributes(cms_qe_video_source_file_video_player_model):
    attr_str = cms_qe_video_source_file_video_player_model._get_attributes_str_to_html(
        ('width', 'height', 'controls', 'autoplay', 'loop', 'muted'))
    assert 'width=500' in attr_str and 'height=500' in attr_str
    assert 'controls' not in attr_str
    assert 'autoplay' in attr_str
    assert 'loop' in attr_str
    assert 'muted' in attr_str


def test__get_attributes_str_to_url_with_default_attributes(cms_qe_video_source_file_video_player_model):
    attr_str = cms_qe_video_source_file_video_player_model._get_attributes_str_to_url(
        ('width', 'height', 'controls', 'autoplay', 'loop', 'muted'))
    assert 'width' not in attr_str and 'height' not in attr_str
    assert 'controls=1' in attr_str
    assert 'autoplay' not in attr_str
    assert 'loop' not in attr_str
    assert 'muted' not in attr_str


@use_data(cms_qe_video_source_file_video_player_model_data=non_default_data)
def test__get_attributes_str_to_url_with_non_default_attributes(cms_qe_video_source_file_video_player_model):
    attr_str = cms_qe_video_source_file_video_player_model._get_attributes_str_to_url(
        ('width', 'height', 'controls', 'autoplay', 'loop', 'muted'))
    assert 'width=500' in attr_str and 'height=500' in attr_str
    assert 'controls' not in attr_str
    assert 'autoplay=1' in attr_str
    assert 'loop=1' in attr_str
    assert 'muted=1' in attr_str
    assert attr_str.count('&') == 4


@use_data(
    cms_qe_video_hosting_video_player_model_data={'video_hosting_service': YOUTUBE,
                                                  'video_url': 'isnotyoutube.com/somevideo', })
def test_clean_youtube_bad_url(cms_qe_video_hosting_video_player_model):
    with pytest.raises(ValidationError) as validation_exception:
        cms_qe_video_hosting_video_player_model.clean()
    validation_exception.match(r'.*YouTube.*')


@use_data(
    cms_qe_video_hosting_video_player_model_data={'video_hosting_service': VIMEO,
                                                  'video_url': 'notvimeo.com/somevideo', })
def test_clean_vimeo_bad_url(cms_qe_video_hosting_video_player_model):
    with pytest.raises(ValidationError) as validation_exception:
        cms_qe_video_hosting_video_player_model.clean()
    validation_exception.match(r'.*Vimeo.*')


@use_data(cms_qe_video_hosting_video_player_model_data={'video_url': 'youtube.com/somevideo',
                                                        'video_hosting_service': YOUTUBE})
def test_clean_youtube_good_urls(cms_qe_video_hosting_video_player_model):
    cms_qe_video_hosting_video_player_model.clean()


@use_data(cms_qe_video_hosting_video_player_model_data={'video_url': 'vimeo.com/somevideo',
                                                        'video_hosting_service': VIMEO})
def test_clean_vimeo_good_urls(cms_qe_video_hosting_video_player_model):
    cms_qe_video_hosting_video_player_model.clean()


@use_data(cms_qe_video_hosting_video_player_model_data={'video_url': 'vimeo.com/somevideo',
                                                        'video_hosting_service': VIMEO,
                                                        'controls': False})
def test_clean_vimeo_disabled_controls(cms_qe_video_hosting_video_player_model):
    with pytest.raises(ValidationError) as validation_exception:
        cms_qe_video_hosting_video_player_model.clean()
    validation_exception.match(r'.*Vimeo.*controls.*')



class TableDoesNotExists(Exception):
    """
    Exception when plugin is configured with table which does not exists
    anymore. For example developer changed name or application providing
    that table is not used anymore.
    """

    def __init__(self, table_name: str) -> None:
        super().__init__()
        self.table_name = table_name



from unittest import mock


def test_not_set_mailchimp():
    with mock.patch('cms_qe_newsletter.external_services.sync.sync_tasks', return_value=[
        (True, 'ok'),
        (None, 'warning'),
        (False, 'error'),
    ]):
        # Import here so mock works correctly.
        from ..management.commands.cms_qe_newsletter_sync import Command

        with mock.patch('cms_qe_newsletter.management.commands.cms_qe_newsletter_sync.logger') as log_mock:
            command = Command()
            command.handle()
            assert log_mock.info.call_args_list == [
                mock.call('Newsletter sync started...'),
                mock.call('ok'),
                mock.call('Newsletter sync finished...'),
            ]
            assert log_mock.warning.call_args_list == [mock.call('warning')]
            assert log_mock.error.call_args_list == [mock.call('error')]



from django import forms
from django.contrib.auth.forms import PasswordResetForm, UserCreationForm
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from .models import User


class RegisterForm(UserCreationForm):
    """
    Main form for registration. Extends base Django`s
    ``UserCreationForm`` by including also email.
    """

    email = forms.EmailField(required=True)

    class Meta:
        model = User
        fields = ('username', 'email', 'password1', 'password2')


class PasswordResetFormWithEmailExistenceCheck(PasswordResetForm):
    """
    Adding verification that the user exists in the database to PasswordResetForm.

    https://github.com/django/django/blob/master/django/contrib/auth/views.py
    """

    def clean_email(self):
        email = self.cleaned_data['email']
        if not User.objects.filter(email=email).exists():
            raise ValidationError(_("Email does not exists"))
        return email



"""
Breadcrumb plugin
#################

Plugin for Django CMS QE providing breadcrumb. It allows include breadcrumb
at any place so it can be moved without support of developers.

Usage
*****

Add plugin from section **Generic** called **Breadcrumb**.

API
***

.. autoclass:: cms_qe_breadcrumb.cms_plugins.BreadcrumbPlugin
    :members:

"""



from django.contrib.auth import get_user_model
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    def handle(self, *args, **options):
        user = get_user_model().objects.filter(is_superuser=True).first()
        if user:
            self.stdout.write('The superuser \'{}\' already exists.'.format(user.username))
        else:
            password = 'admin'
            user = get_user_model().objects.create_superuser('admin', 'admin@example.com', password)
            self.stdout.write(
                """A new superuser with the following data was created:
                login:    {}
                email:    {}
                password: {}""".format(user.username, user.email, password)
            )



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class NewsletterConfig(AppConfig):
    name = 'cms_qe_newsletter'
    verbose_name = _('CMS QE Newsletter')




from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from django.utils.translation import ugettext_lazy as _

from .forms import TablePluginForm
from .models import TablePluginModel


@plugin_pool.register_plugin
class TablePlugin(CMSPluginBase):
    """
    CMS plugin allowing to add dynamically configured listing of any model.
    """

    form = TablePluginForm
    model = TablePluginModel
    name = _('Table')
    render_template = 'cms_qe/table/table.html'
    cache = False

    def render(self, context: dict, instance: TablePluginModel, placeholder) -> dict:
        context = super().render(context, instance, placeholder)
        if instance.table_exists and instance.columns_exist:
            page_param_name = 'page_{}'.format(instance.pk)
            page = context['request'].GET.get(page_param_name, None)
            context.update({
                'header': instance.get_header(),
                'items': instance.get_items(page),
                'page_param_name': page_param_name,
            })
        return context



from django.contrib.auth.forms import AuthenticationForm, UserCreationForm
from django.utils.translation import ugettext_lazy as _

from cms.models.pluginmodel import CMSPlugin
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool


@plugin_pool.register_plugin
class LoginButtonPlugin(CMSPluginBase):
    """
    CMS plugin showing link to login or logout.
    """

    module = _('Auth')
    name = _('Login button')
    render_template = 'cms_qe/auth/login_button_plugin.html'
    text_enabled = True
    cache = False


@plugin_pool.register_plugin
class LoginFormPlugin(CMSPluginBase):
    """
    CMS plugin allowing the user to log into site.
    """

    module = _('Auth')
    name = _('Login form')
    render_template = 'cms_qe/auth/login_form_plugin.html'
    cache = False

    def render(self, context: dict, instance: CMSPlugin, placeholder) -> dict:
        context = super().render(context, instance, placeholder)
        context.update({
            'form': AuthenticationForm(),
        })
        return context


@plugin_pool.register_plugin
class RegisterFormPlugin(CMSPluginBase):
    """
    CMS plugin allowing the user to register into site.
    """

    module = _('Auth')
    name = _('Register form')
    render_template = 'cms_qe/auth/register_plugin.html'
    cache = False

    def render(self, context: dict, instance: CMSPlugin, placeholder) -> dict:
        context = super().render(context, instance, placeholder)
        context.update({
            'form': UserCreationForm(),
        })
        return context



from cms_qe_auth.utils import pk_to_uidb64
from cms_qe_auth.models import User
from django.urls import reverse
from pytest_data import use_data


@use_data(user_data={'username': 'testuser', 'password': 'testpass'})
def test_login(client, user):
    res = client.post('/en/auth/login/', {'username': 'testuser', 'password': 'testpass'})
    assert res.status_code == 302


def test_register(client):
    assert not len(User.objects.filter(username='testuser'))
    res = client.post('/en/auth/register/', {
        'username': 'testuser',
        'password1': 'testpass',
        'password2': 'testpass',
        'email': 'testuser@example.com',
    })
    assert res.status_code == 302
    user = User.objects.get(username='testuser')
    assert user.email == 'testuser@example.com'


@use_data(user_data={'is_active': False})
def test_activation(client, user):
    assert not user.is_active
    token = user._generate_activation_token()
    uid = pk_to_uidb64(user.pk)
    url = reverse('activate', kwargs={'uidb64': uid, 'token': token})
    response = client.get(url)
    user.refresh_from_db()
    assert user.is_active
    # Test automatic login
    assert response.context['user'].is_authenticated



"""
Django CMS QE
#############

Django CMS Quick & Easy provides all important modules to run new page
without a lot of coding. Aims to do it very easily and securely.

API
***

Settings
========

.. automodule:: cms_qe.settings
   :members:

Views
=====

.. automodule:: cms_qe.views.errors
   :members:

.. automodule:: cms_qe.views.monitoring
   :members:

.. automodule:: cms_qe.views.security
   :members:

Template tags and filters
=========================

.. automodule:: cms_qe.templatetags.cms_qe_filters
   :members:

Other
=====

.. automodule:: cms_qe.export
   :members:

.. automodule:: cms_qe.staticfiles
   :members:

.. automodule:: cms_qe.utils
   :members:

"""

from . import django_111_compatibility




from django.http import HttpRequest, JsonResponse

from . import utils


def get_table_choices(request: HttpRequest) -> JsonResponse:
    """
    After choosing table, form has to show available columns. It's done
    by JavaScript to call this view to get that data. URL expect one
    GET parameter called ``table``. It's because it's easier to dynamicly
    change in JavaScript.

    Output format is same as from :any:`cms_qe_table.utils.get_table_choices`.
    """
    table = request.GET.get('table', '')
    return JsonResponse(utils.get_table_choices(table))



"""
Configuration for development.

Disables all security options.
"""

import os
from typing import List  # pylint: disable=unused-import

from .base import *  # pylint: disable=wildcard-import,unused-wildcard-import

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.11/howto/deployment/checklist/

DEBUG = True

META_SITE_PROTOCOL = 'http'

SECRET_KEY = '^xzhq0*q1+t0*ihq^^1wuyj3i%y#(38b7d-vlpkm-d(=!^uk6x'

SESSION_COOKIE_SECURE = False

SECURE_HSTS_SECONDS = 0

ALLOWED_HOSTS = ['localhost']

INSTALLED_APPS += [
    'debug_toolbar',
    'django_extensions',
]

MIDDLEWARE += [
    'debug_toolbar.middleware.DebugToolbarMiddleware',
]

BASE_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), '..')

# Logging
# https://docs.djangoproject.com/en/1.11/topics/logging/

LOGGING['loggers'] = {
    '': {
        'level': 'INFO',
        'handlers': ['console'],
    },
}

# Caching
# https://docs.djangoproject.com/en/1.11/topics/cache/

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
    }
}

# Database
# https://docs.djangoproject.com/en/1.11/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, '..', 'db.sqlite3'),
        'TEST': {
            'NAME': ':memory:',
        },
    }
}

EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'



"""
Base settings for Django templates.
"""

# Possible to change to another but all apps have to support that other one.
# For now is counted only with Bootstrap which has the biggest support.
# More info: https://github.com/aldryn/aldryn-boilerplates/
ALDRYN_BOILERPLATE_NAME = 'bootstrap3'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'OPTIONS': {
            'context_processors': [
                # Django's defaults.
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django.template.context_processors.i18n',  # Needed by Django CMS.

                # Django CMS's core context processors.
                'cms.context_processors.cms_settings',
                'sekizai.context_processors.sekizai',  # Static file management for template blocks.
                'aldryn_boilerplates.context_processors.boilerplate',  # Static file management for Aldryn Boilerplates.

                # Other Django's modules.
                'constance.context_processors.config',
            ],
            'loaders': [
                'django.template.loaders.filesystem.Loader',
                'aldryn_boilerplates.template_loaders.AppDirectoriesLoader',  # Loader of Aldryn Boilerplate templates.
                'django.template.loaders.app_directories.Loader',
            ],
        },
    },
]




class CreatePagePage:
    def __init__(self, driver):
        self._driver = driver

    def open(self):
        self._driver.go_to('/admin/cms/page/add/')
        return self

    def create_page(self, title, slug=''):
        # Title will automatically set slug which we want to force.
        self._driver.get_elm(tag_name='form').fill_out({
            'title': title,
        })
        self._driver.get_elm(tag_name='form').fill_out_and_submit({
            'slug': slug,
        })



"""
Internationalization settings for Django app.
"""

from django.utils.translation import ugettext_lazy as _

# Internationalization
# https://docs.djangoproject.com/en/1.11/topics/i18n/

LANGUAGE_CODE = 'en'

LANGUAGES = (
    ('en', _(u'English')),
    ('cs', _(u'Czech')),
)

LOCALE_PATHS = (
    'locale',
)

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True



from cms.models.pluginmodel import CMSPlugin
from django.utils.translation import ugettext as _
from django.db import models


class MenuPluginModel(CMSPlugin):
    HORIZONTAL = 1
    VERTICAL = 2
    ORIENTATION_CHOICES = (
        (HORIZONTAL, _('Horizontal')),
        (VERTICAL, _('Vertical'))
    )
    orientation = models.IntegerField(choices=ORIENTATION_CHOICES, default=HORIZONTAL)

    # reverse_id in page model
    start_page = models.CharField(
        max_length=50,
        blank=True,
        help_text=_('ID of page in advanced settings.'),
    )
    end_level = models.IntegerField(
        default=2,
        verbose_name=_('End level'),
        help_text=_('''
            Specify at which level the navigation should stop.\n
            Keep in mind that Bootstrap by default does not support dropdown submenu.
            Any deep dropdown menu is not very well accessible from mobile devices.
        '''),
    )

    submenu_in_inactive_item = models.BooleanField(
        default=True,
        verbose_name=_('Submenu for inactive items'),
    )

    def __str__(self):
        return _('Menu')

    @property
    def is_horizontal(self) -> bool:
        return self.orientation == self.HORIZONTAL

    @property
    def is_vertical(self) -> bool:
        return self.orientation == self.VERTICAL

    @property
    def extra_inactive(self) -> int:
        """
        Third parameter of tag `show_menu` of Django CMS. Specifies
        how many sub items should be displayed when item is not active.

        Admin can choose if he wants to show everything or nothing.
        """
        if not self.submenu_in_inactive_item:
            return 0
        return self.end_level

    @property
    def extra_active(self) -> int:
        """
        Fourth parameter of tag `show_menu` of Django CMS. Specifies
        how many sub items should be displayed when item is active.

        For vertical menu is good for users to show only one more
        level, for horizontal with dropdown is possible to show all
        to same deep level admin want to users to be able to go.
        """
        if self.is_horizontal:
            return self.end_level
        return 1



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-20 09:54
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import jsonfield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
    ]

    operations = [
        migrations.CreateModel(
            name='TablePluginModel',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='cms_qe_table_tablepluginmodel', serialize=False, to='cms.CMSPlugin')),
                ('table', models.CharField(max_length=100, verbose_name='Table to show')),
                ('columns', jsonfield.fields.JSONField(default=dict, verbose_name='Columns to show')),
                ('paging_show', models.BooleanField(default=True, verbose_name='Show paging')),
                ('paging_per_page', models.IntegerField(default=20, verbose_name='How many items per page when paging')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]




from django.template import Library, TemplateDoesNotExist, loader


register = Library()


@register.filter
def cms_qe_table_value(value: str) -> str:
    """
    Django template filter to customize displaying of values by their type.
    If value is of type bool, then is used template ``cms_qe/table/table_value_bool.html``.
    Every boilerplate or concrete app can customize this. When no template exists
    for given value type standard value represenation is used.
    """
    value_type = type(value).__name__
    template_name = 'cms_qe/table/table_value_{}.html'.format(value_type)
    try:
        template = loader.get_template(template_name)
    except TemplateDoesNotExist:
        return value
    else:
        context = {
            'value': value,
        }
        return template.render(context)



#!/usr/bin/env python3

import os

import setuptools


if __name__ == '__main__':
    readme_filename = os.path.join(os.path.dirname(__file__), 'README.md')
    with open(readme_filename, encoding='utf-8') as readme_file:
        readme = readme_file.read()
    setuptools.setup(
        name='django-cms-qe',
        version='1.0',
        packages=setuptools.find_packages(exclude=[
            '*.tests',
            '*.tests.*',
            'tests.*',
            'tests',
            'test_utils.*',
            'test_utils',
            '*.migrations',
            '*.migrations.*',
        ]),
        include_package_data=True,
        description=(
            'Django CMS Quick & Easy provides all important modules to run new page without'
            'a lot of coding. Aims to do it very easily and securely.'
        ),
        long_description=readme,
        url='https://websites.pages.labs.nic.cz/django-cms-qe',
        author='CZ.NIC, z.s.p.o.',
        author_email='kontakt@nic.cz',
        license='BSD License',

        # All versions are fixed just for case. Once in while try to check for new versions.
        install_requires=[
            'aldryn-boilerplates==0.7.5',
            'aldryn-bootstrap3==1.2.2',
            'aldryn-forms==2.2.9',
            'argon2_cffi==16.3.0',
            'cmsplugin-filer==1.1.3',
            'Django>=1.11.0,<2.0',
            'django-axes==2.3.3',
            'django-bootstrap-form==3.3',
            'django-cms==3.4.5',
            'django-constance[database]==2.0.0',
            'django-csp==3.3',
            'django-filer==1.2.8',
            'django-import-export==0.5.1',
            'django-jsonfield==1.0.1',
            'django-settings==1.3.12',
            'djangocms-googlemap==1.1.1',
            'djangocms-text-ckeditor==3.5.1',
            'djangocms-attributes-field>=0.1.1',  # Used by cms-qe-video
            'djangocms-inline-comment==0.0.2',
            'easy-thumbnails==2.4.2',  # Used by Django Filer, v2.4.2 works with Django 1.11
            'mailchimp3==2.0.15',
            'python-memcached==1.58',
            'pytz==2017.2',
        ],
        # Do not use test_require or build_require, because then it's not installed and is
        # able to be used only by setup.py util. We want to use it manually.
        # Actually it could be all in dev-requirements.txt but it's good to have it here
        # next to run dependencies and have it separated by purposes.
        extras_require={
            'dev': [
                'django-debug-toolbar==1.8',
                'django_extensions==1.9.6',
            ],
            'test': [
                'mypy==0.540',
                'pylint==1.7.4',
                'pylint-django==0.7.2',
                'pytest==3.2.3',
                'pytest-data==0.4',
                'pytest-django==3.1.2',
                'pytest-env==0.6.2',
                'pytest-pythonpath==0.7.1',
                'pytest-sugar==0.9.0',
                'pytest-watch==4.1.0',
                'pyvirtualdisplay==0.2.1',
                'webdriverwrapper==2.5.0',
            ],
            'build': [
                'Sphinx==1.6.5',
            ],
            'psql': [
                'psycopg2==2.7.3',
            ],
            'mysql': [
                'mysqlclient',
            ],
        },

        classifiers=[
            'Development Status :: 5 - Production/Stable',
            'Environment :: Web Environment',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: BSD License',
            'Operating System :: OS Independent',
            'Natural Language :: English',
            'Programming Language :: Python :: 3',
            'Topic :: Software Development :: Libraries :: Application Frameworks',
            'Framework :: Django',
            'Framework :: Django :: 1.10',
            'Framework :: Django :: 1.11',
        ],
        keywords=['django', 'cms'],
    )




from typing import Any

from django import forms
from django.utils.html import mark_safe
from django.template import loader
from django.urls import reverse

from .models import TablePluginModel
from .utils import get_models_choices


class TableWidget(forms.Select):
    """
    Custom widget for table select which reloads choices for columns.
    """

    def render(self, name: str, value: str, attrs: dict = None, renderer: Any = None) -> Any:
        select = super().render(name, value, attrs, renderer)
        context = {
            'id': attrs['id'],
            'select': select,
            'value': value,
            'url': reverse('get_table_choices'),
        }
        template = loader.get_template('cms_qe/table/table_widget.html').render(context)
        return mark_safe(template)


class ColumnsWidget(forms.SelectMultiple):
    """
    Custom widget for columns select which provides needed information for
    :any:`cms_qe_table.forms.TableWidget`. Please use both in the same form.
    """

    def render(self, name: str, value: str, attrs: dict = None, renderer: Any = None):
        select = super().render(name, value, attrs, renderer)
        context = {
            'id': attrs['id'],
            'select': select,
            'value': value,
        }
        template = loader.get_template('cms_qe/table/columns_widget.html').render(context)
        return mark_safe(template)


class MultipleChoiceField(forms.MultipleChoiceField):
    """
    Same as Django's ``MultipleChoiceField`` but accept any value because it's
    used without any choices.
    """

    def valid_value(self, value: Any) -> bool:
        return True


class TablePluginForm(forms.ModelForm):
    """
    Configuration form for plugin Table for Django CMS QE.
    """

    table = forms.ChoiceField(choices=get_models_choices(), widget=TableWidget)
    columns = MultipleChoiceField(widget=ColumnsWidget)

    class Meta:
        model = TablePluginModel
        fields = ['table', 'columns', 'paging_show', 'paging_per_page']



from django.contrib.auth import get_user_model
from django.utils.encoding import force_bytes, force_text
from django.utils.http import urlsafe_base64_decode, urlsafe_base64_encode

# pylint:disable=invalid-name

def pk_to_uidb64(pk):
    return urlsafe_base64_encode(force_bytes(pk))


def uidb64_to_pk(uidb64):
    return force_text(urlsafe_base64_decode(uidb64))


def get_user_by_uidb64(uidb64):
    User = get_user_model()
    try:
        pk = uidb64_to_pk(uidb64)
        user = User.objects.get(pk=pk)
    except(TypeError, ValueError, OverflowError, User.DoesNotExist):
        user = None
    return user



from .client import MailChimpClient, MailchimpIsNotSetException
from ...models import SERVICE_MAILCHIMP, MailingList


def sync_mailing_lists():
    try:
        client = MailChimpClient()
    except MailchimpIsNotSetException:
        return

    result = client.get_lists()
    for mailing_lits in result['lists']:
        if not MailingList.objects.filter(external_service=SERVICE_MAILCHIMP, external_id=mailing_lits['id']).count():
            MailingList.objects.create(
                name=mailing_lits['name'],
                external_service=SERVICE_MAILCHIMP,
                external_id=mailing_lits['id'],
            )


def sync_subscribe(mailing_list_id, email, first_name, last_name):
    return MailChimpClient().subscribe(mailing_list_id, email, first_name, last_name)


def sync_unsubscribe(mailing_list_id, email):
    MailChimpClient().unsubscribe(mailing_list_id, email)



from typing import List, Union

from cms.models.pluginmodel import CMSPlugin
from django.core.paginator import Paginator, Page, EmptyPage, PageNotAnInteger
from django.db import models
from django.db.models.base import ModelBase
from django.utils.translation import ugettext as _
from jsonfield import JSONField

from .exceptions import TableDoesNotExists
from .utils import get_model_by_table


class TablePluginModel(CMSPlugin):
    """
    Configuration model for plugin Table for Django CMS QE. Possibility to specify
    table, columns, order, filtering etc. etc.
    """

    table = models.CharField(max_length=100, verbose_name=_('Table to show'))
    columns = JSONField(verbose_name=_('Columns to show'))
    paging_show = models.BooleanField(default=True, verbose_name=_('Show paging'))
    paging_per_page = models.IntegerField(default=20, verbose_name=_('How many items per page when paging'))

    def __str__(self):
        if self.table_exists:
            return '{m.app_label} / {m.object_name}'.format(m=self.model._meta)
        return _('Non existing table {}.').format(self.table)

    @property
    def table_exists(self) -> bool:
        """
        Returns if table exists. When programmer move model or rename table
        without change in this configuration, it will not be available anymore.
        """
        try:
            get_model_by_table(self.table)
            return True
        except TableDoesNotExists:
            return False

    @property
    def columns_exist(self) -> bool:
        """
        Returns if all columns exists. When programmer rename column
        without change in this configuration, it will not be available anymore.
        """
        if not self.table_exists:
            return False
        model = self.model
        # pylint:disable=not-an-iterable
        if not all(hasattr(model, column) for column in self.columns):
            return False
        return True

    @property
    def model(self) -> ModelBase:
        """
        Returns model for configured table.
        """
        return get_model_by_table(self.table)

    def get_header(self) -> List[str]:
        """
        Returns header for table with ``verbose_name`` of fields if exists.
        In other cases at least uses name of that field.
        """
        def f(column):
            field = self.model._meta.get_field(column)
            return getattr(field, 'verbose_name', field.name)
        return [f(column) for column in self.columns]  # pylint: disable=not-an-iterable

    def get_items(self, page: Union[int, str] = None) -> Page:
        """
        Returns items for table without header. It's simply list (items)
        of lists (columns), not whole objects. Header is not included,
        for that use :any:`TablePluginModel.get_header`.
        """
        items_list = self.model.objects.all()
        if not self.paging_show:
            return self._get_items(items_list)

        paginator = Paginator(items_list, self.paging_per_page)

        try:
            items = paginator.page(page)
        except PageNotAnInteger:
            items = paginator.page(1)
        except EmptyPage:
            items = paginator.page(paginator.num_pages)

        # Evaluate query and return only needed columns.
        items.object_list = self._get_items(items)
        return items

    def _get_items(self, items: List[ModelBase]) -> List[List[str]]:
        return [
            [getattr(item, column, '') for column in self.columns]  # pylint: disable=not-an-iterable
            for item in items
        ]



import pytest
from django.contrib.messages.storage.fallback import FallbackStorage
from django.test import RequestFactory
from pytest_data import get_data


@pytest.fixture
def post_request_empty(request):
    rf = RequestFactory()
    rq = rf.post('', get_data(request, 'post_request_empty_data', {}))
    setattr(rq, 'session', True)
    messages = FallbackStorage(rq)
    setattr(rq, '_messages', messages)
    return rq
